package ch.programmieraufgaben.planetensortieren.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.lang.reflect.Field;

import javax.swing.*;
import javax.swing.border.BevelBorder;

import ch.programmieraufgaben.planetensortieren.model.Himmelskoerper;
import ch.programmieraufgaben.planetensortieren.model.Planet;


/**
 * Zeige einen Planeten mit allen Attributen.
 * 
 * @author Philipp Gressly (phi AT gressly DOT ch)
 */
public class PlanetPanel extends JPanel {
    public static final int MARK_NONE    = 0;
    public static final int MARK_GREATER = 1;
    public static final int MARK_LESS    = 2;
    public static final int MARK_MOVING  = 3;

    Planet                  planet;
    ScaleableImagePanel     sip;
    InfoPanel               infoPanel;
    GUI                     gui;

    public PlanetPanel(Planet planet, GUI gui){
        this.gui = gui;
        setBackground(Color.BLACK);
        setForeground(Color.WHITE);
        infoPanel = new InfoPanel(gui);
        setLayout(new BorderLayout());
        markVisually(MARK_NONE);
        this.planet = planet;
        sip = new ScaleableImagePanel("img/" + planet.id + ".png");
        add(sip);
        
        fillInfoPanel();  }

    private void fillInfoPanel() {
        Class<? extends Himmelskoerper> c = this.planet.getClass();
        Field[] fields = c.getFields();
        for (Field f : fields) {
            String name = f.getName();
            Object value = null;
            try {
                value = f.get(this.planet); } 
            catch (Exception e) {
                System.err.println("Error in PlanetPanel: " + e);
                e.printStackTrace(); }
            if(! "id".equals(name)) {
              infoPanel.addInfo(name, value.toString()); }
        }
        infoPanel.setSortCriteria();
        add(infoPanel, BorderLayout.SOUTH);   }
    

    /**
     * Sortierkriterium  (= Attriubt in der Klasse "Planet")
     */
    public void setKriterium(String kriterium) {
        infoPanel.setSortCriteria();
        validate();   }

    
    /**
     * Planeten visuell aufheitern.
     * @param mark
     */
    public void markVisually(int mark) {
        Color c = null;
        switch(mark) {
            case MARK_GREATER: c = Color.RED;    break;
            case MARK_LESS   : c = Color.GREEN;  break;
            case MARK_MOVING : c = Color.YELLOW; break;   }
        if(null != sip) {
            if(null != c) {
                sip.lighten(c);  }
        }
        if(MARK_NONE == mark) {
            c = Color.GRAY;
            if(null != sip) {
                sip.unlighten();   }
        }
        setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED, c
                .brighter(), c.darker()));  }

}  // end of class PlanetPanel
