package ch.programmieraufgaben.planetensortieren.gui;

import java.awt.*;
import java.awt.event.*;
import java.util.HashMap;

import javax.swing.JPanel;

import ch.programmieraufgaben.planetensortieren.model.Planet;
import ch.programmieraufgaben.planetensortieren.model.Sonnensystem;


import static eu.gressly.hw.iteration.Sequencer.seq;

/**
 * Zeige die 8 Planeten in einem Panel.
 * Das Panel ist drei mal so hoch, wie die Planeten. 
 * Das ist daher so, weil beim Tausch der Planeten
 * diese oben, bzw. untendurch vertauscht. 
 * 
 * @author Philipp Gressly (phi AT gressly DOT ch)
 */
public class SonnensystemPanel extends JPanel {
  
    Sonnensystem sonnensystem;
    HashMap<Planet, PlanetPanel> planetenPanels = new HashMap<Planet, PlanetPanel>();
    GUI gui;


    public SonnensystemPanel(Sonnensystem sonnensystem, GUI gui) {
        this.gui = gui;
        this.sonnensystem = sonnensystem;
        
        setLayout(null);
        this.setBackground(Color.BLACK);
        for(Planet p : sonnensystem.getPlaneten()) {
            PlanetPanel pp = new PlanetPanel(p, gui);
            planetenPanels.put(p, pp);
            add(pp);    }
        addComponentListener(new ComponentAdapter() {
            public void componentResized(ComponentEvent e) {
               layoutPlaneten();  }
            public void componentShown(ComponentEvent e) {
                layoutPlaneten();  }
        });
        layoutPlaneten();   }
    
    
    void layoutPlaneten() {
        int w = getWidth() / planetenPanels.size();
        int h = getHeight() / 3;
        int y = h;
        int x = 0;
        // folgendes kann nicht via "for über planetenPanels" erreicht werden: Reihenfolge in der HashMap ist willkürlich!
        for(Planet p : sonnensystem.getPlaneten()) {
            PlanetPanel pp = planetenPanels.get(p);
            pp.setBounds(x, y, w, h);
            pp.validate();
            x = x + w;  }
        validate();  }
    
    
    PlanetPanel getPlanetPanel(int pos) {
        Planet p = sonnensystem.getPlaneten().get(pos);
        return planetenPanels.get(p);  }
    
    
    public void tausche(int pos1, int pos2) { 
        PlanetPanel pp1 = getPlanetPanel(pos1);
        PlanetPanel pp2 = getPlanetPanel(pos2);
       try {
           for(int i : seq(1, 4)) { // i is unused
               pp1.markVisually(PlanetPanel.MARK_MOVING);
               pp2.markVisually(PlanetPanel.MARK_MOVING);
               Thread.sleep(gui.getVitesse()/4);
               pp1.markVisually(PlanetPanel.MARK_NONE);
               pp2.markVisually(PlanetPanel.MARK_NONE);
               Thread.sleep(gui.getVitesse()/4);
           }
           
           pp1.markVisually(PlanetPanel.MARK_MOVING);
           pp2.markVisually(PlanetPanel.MARK_MOVING);
           
           bezierMove(pp1, pp2);
           
           Thread.sleep(gui.getVitesse());   } 
       catch (InterruptedException e) {
           stopSorterThread();
           Thread.currentThread().stop(); } 
       finally {
         pp1.markVisually(PlanetPanel.MARK_NONE);
         pp2.markVisually(PlanetPanel.MARK_NONE);
         layoutPlaneten();   }    
    }
    
    
    /**
     * Zwei Bezier-Kurven (eine oben, die andere unten an den Planeten vorbei) vertauschen
     * zwei Panels auf "weichen" Kurven. 
     */
    private void bezierMove(PlanetPanel pp1, PlanetPanel pp2) throws InterruptedException {
        if(pp1 == pp2) {
            return;   }
        this.setComponentZOrder(pp1, 0);
        this.setComponentZOrder(pp2, 1);
        int hd = getHeight() / 3; // height of one PlanetPanel inside the SonnensystemPanel
        
        BezierGrad3 b2dP1P2x = new BezierGrad3(pp1.getX(),pp1.getX(), pp2.getX(), pp2.getX());
        BezierGrad3 b2dP2P1x = new BezierGrad3(pp2.getX(),pp2.getX(), pp1.getX(), pp1.getX());
        
        BezierGrad3 b2dP1P2y = new BezierGrad3(hd, -hd/3,  -hd/3,   hd);
        BezierGrad3 b2dP2P1y = new BezierGrad3(hd,  hd*7/3, hd*7/3, hd);
    
        for(int i = 0; i < 65; i++) {
           Thread.sleep(gui.getVitesse() / 64 + 1);
           pp1.setLocation(b2dP1P2x.get(i), b2dP1P2y.get(i));
           pp2.setLocation(b2dP2P1x.get(i), b2dP2P1y.get(i));  }
     }

    
    /**
     * Markiere grün, falls die zu vergleichenden Planeten bereits in Reihenfolge sind
     * und rot andernfalls.
     */
    public void vergleich(int pos1, int pos2, boolean bereitsSortiern) {
        PlanetPanel pp1 = getPlanetPanel(pos1);
        PlanetPanel pp2 = getPlanetPanel(pos2);
        int mark;
        if(bereitsSortiern) {
            mark = PlanetPanel.MARK_LESS; }
        else {
            mark = PlanetPanel.MARK_GREATER; }
        pp1.markVisually(mark);
        pp2.markVisually(mark);
            
        try {
            Thread.sleep(gui.getVitesse());
        } catch (InterruptedException e) {
            stopSorterThread();
            Thread.currentThread().stop();
        } finally {
          pp1.markVisually(PlanetPanel.MARK_NONE);
          pp2.markVisually(PlanetPanel.MARK_NONE);
        }
    }
    
    /**
     * How to stopp (without the "stop()" - Metohd ?????????????????????
     */
    private void stopSorterThread() {
        this.gui.stopSorting();  }
    

    /**
     * Set in every Planet
     */
    public void setKriterium(String kriterium) {
        for(PlanetPanel pp : planetenPanels.values()) {
            pp.setKriterium(kriterium);   }
    }
    
} // end of class SonnensystemPanel
