package ch.programmieraufgaben.planetensortieren.model;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.ResourceBundle;
import java.util.StringTokenizer;

import ch.programmieraufgaben.planetensortieren.gui.GUI;

  /**
   * Menge aller Planeten
   * @author Philipp Gressly (phi AT gressly DOT ch)
   */
  public class Sonnensystem {
    ArrayList<Planet> planeten = new ArrayList<Planet>();
    private GUI gui;
   
    public Sonnensystem(GUI gui) {
      this.gui = gui;
      
      readData(); }
  
    
    private void readData() {
        ResourceBundle rb = ResourceBundle.getBundle("ch.programmieraufgaben.planetensortieren.model.model");
        StringTokenizer st = new StringTokenizer(rb.getString("planets"), ",");
        while(st.hasMoreTokens()) {
            String planet = st.nextToken();
            Planet p;
            try {
                p = getPlanet(rb, planet);
                planeten.add(p);
            } catch (Exception e) {
                System.out.println(rb.getString("readData.exception"));
                e.printStackTrace(); }
        }
    }
    
   
    private Planet getPlanet(ResourceBundle rb, String planet) throws IllegalArgumentException, IllegalAccessException {
        Planet p = new Planet(planet, this.gui);
        Class<? extends Himmelskoerper> planetClass = Planet.class;
        Field[] planetenFields = planetClass.getFields();
        for(Field f : planetenFields) {
            if(! f.getName().equals("id")) {
                if(f.getType().equals(String.class)) {
                    String fieldValue = rb.getString(planet + "." + f.getName());
                    f.set(p, fieldValue);
                }
                if(f.getType().equals(double.class)) {
                    String fieldString = rb.getString(planet + "." + f.getName());
                    Double fieldDouble = Double.parseDouble(fieldString);
                    f.set(p, fieldDouble);
                }
            }
        }
        return p;
    }

    public int size() {
      return planeten.size();  }

   /**
    * Vertausche die beiden Planeten an pos1 mit pos2.
    */
    public void tausche(int pos1, int pos2) {
      tauscheImArray(pos1, pos2);  
      gui.tausche(pos1, pos2);  }

   /**
    * Vertausche im Array der Planeten
    */
    private void tauscheImArray(int pos1, int pos2) {
      Planet tempPlanet1 = planeten.get(pos1);
      Planet tempPlanet2 = planeten.get(pos2);
      planeten.set(pos1, tempPlanet2);
      planeten.set(pos2, tempPlanet1); }

 
    public ArrayList<Planet> getPlaneten() {
      return planeten;  }

    public boolean istKleiner(int pos1, int pos2) {
      boolean istKleiner;
      Planet p1 = planeten.get(pos1);
      Planet p2 = planeten.get(pos2);
      if(gui.isAscendig()) {
          istKleiner = p1.istKleinerAls(p2);  }
      else { 
          istKleiner = p2.istKleinerAls(p1);  }
      gui.vergleich(pos1, pos2, istKleiner);
      return istKleiner;  }

    
    public void mische() {
      int inPos = 0;
      while(inPos < planeten.size()) {
        int rndPos = rndPos(inPos, planeten.size() - 1);
        tauscheImArray(inPos, rndPos);
        inPos = inPos + 1;  }
    }


   // zahl zwischen min und max (beide inklusive)
   int rndPos(int min, int max) {
     return ((int) (Math.random() * (max - min + 1))) + min; }
   
 
} // end of class Sonnensystem